<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\MAN\Services\GeraCuponsMastersafService;
use Zeedhi\Framework\DTO\Response\Message;
use \Exception as Exception;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO\Response\Error;

class GeraCuponsMastersaf extends CrudWithParameterBag {
    
    const dataSourceCupons         = 'vnd16000_geraCuponsMastersaf';
    const dataSourceSafx201        = 'vnd16000_geraCuponsMastersaf_safx201';
    const dataSourceSafx202        = 'vnd16000_geraCuponsMastersaf_safx202';
    const dataSourceSafx07         = 'vnd16000_geraCuponsMastersaf_safx07';
    const dataSourceSafx08         = 'vnd16000_geraCuponsMastersaf_safx08';
    protected $parameterBagCupons  = array('NRORG', 'CDFILIAL');
    protected $parameterBagSafx    = array('CDFILIAL', 'DTINICIAL', 'DTFINAL', 'IDTIPOCAIXA');
    protected $crudService;
    protected $utilService;
    
              
    public function __construct(ParameterBag $parameterBag, Manager $dataSourceManager, $crudService, Utilities\Util $utilService, EntityManager $entityManager, GeraCuponsMastersafService $geraCuponsMastersafService) {
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->crudService = $crudService;
        $this->utilService = $utilService;
        $this->entityManager = $entityManager;
        $this->geraCuponsMastersafService = $geraCuponsMastersafService;
    }
    
    public function findCupons(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceCupons;
        $this->parameterBagColumns = $this->parameterBagCupons;
        
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if(isset($conditions['CDFILIAL']) && $conditions['CDFILIAL'] !== 'T') {
            $filiais = '_';
            foreach($conditions['CDFILIAL'] as $filial) {
                $filiais = $filiais.$filial.'_';
            }
            
            $conditions['CDFILIAL'] = $filiais;
        } else {
            $conditions['CDFILIAL'] = 'T';
        }
        
        unset($conditions['DATA_EMISSAO']);
        unset($conditions['IDTIPOCAIXA']);
        
        $dataSourceFilter = array();
            
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        parent::find($newRequest, $response);     
    }
    
    public function findSafx201(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceSafx201;
        $this->parameterBagColumns = $this->parameterBagSafx;
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if($conditions['IDTIPOCAIXA'] == 'S'  || $conditions['IDTIPOCAIXA'] == 'T') {
            parent::find($request, $response);
        }
    }
    
    public function findSafx202(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceSafx202;
        $this->parameterBagColumns = $this->parameterBagSafx;
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if($conditions['IDTIPOCAIXA'] == 'S'  || $conditions['IDTIPOCAIXA'] == 'T') {
            parent::find($request, $response);
        }
    }
    
    public function findSafx07(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceSafx07;
        $this->parameterBagColumns = $this->parameterBagSafx;
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if($conditions['IDTIPOCAIXA'] == 'N'  || $conditions['IDTIPOCAIXA'] == 'T') {
            parent::find($request, $response);
        }
    }
    
    public function findSafx08(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceSafx08;
        $this->parameterBagColumns = $this->parameterBagSafx;
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if($conditions['IDTIPOCAIXA'] == 'N'  || $conditions['IDTIPOCAIXA'] == 'T') {
            parent::find($request, $response);
        }
    }
    
    public function verificaDuplicidadeCupons(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $this->utilService->getParamsAsArray($request);
        
        try {
            $retorno = $this->geraCuponsMastersafService->verificaDuplicidadeCupons($conditions['CDFILIAL'], $conditions['DTINICIAL'], $conditions['DTFINAL']);
            $response->addDataSet(new DataSet('vnd16000_verificaDuplicidade', $retorno));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
   
    public function geraCuponsMastersaf(DTO\Request\Filter $request, DTO\Response $response) {
        $dataSet = 'vnd16000_geraCuponsMastersaf/execute';
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $conection = $this->entityManager->getConnection();
        $conection->beginTransaction();
        
        if($conditions['IDTIPOCAIXA'] == 'T') {
            $conditions['IDTIPOCAIXA'] = 'S_N';
        }
        
        if($conditions['CDFILIAL'] == 'T') {
            $conditions['CDFILIAL'] = $this->geraCuponsMastersafService->getFiliais();
        }
        
        try {
            foreach($conditions['CDFILIAL'] as $adqFilial) {
                $this->executaEXPCUPMASTERSAF($adqFilial, $conditions['DTINICIAL'], $conditions['DTFINAL'], $conditions['IDTIPOCAIXA'], $conection);
            }
            $conection->commit();
            $response->addDataSet(new DataSet($dataSet, $conditions));
        } catch (\Exception $e){
            $conection->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function executaEXPCUPMASTERSAF($CDFILIAL, $DTINICIAL, $DTFINAL, $IDTIPOCAIXA, $conection){
	    try{
            $procedure = new StoredProcedure($conection,'EXPCUPMASTERSAF');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT,$CDFILIAL, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT,$DTINICIAL, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT,$DTFINAL, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTIPOCAIXA', Param::PARAM_INPUT,$IDTIPOCAIXA, Param::PARAM_TYPE_STR));
            $result = $procedure->execute();
        }catch (\Exception $e){
            throw $e;
        }
	}
}
